<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class ResumoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getSituacoesFreq(
        array $params
    ) {
        $sql = <<<SQL
            SELECT
                SF.NRSITUFREQUENCIA,
                SF.NRORG,
                SF.DSSITUFREQUENCIA,
                SF.DSABREVIACAO,
                (DSSITUFREQUENCIA || (CASE COUNT(EVM.NREVENTOM)
                    WHEN 0 THEN ''
                    ELSE ' [' || LISTAGG(NMEVENTOH, ', ') WITHIN GROUP (ORDER BY NMEVENTOH) || ']'
                END)) DSSITUEVENTOS,
                LISTAGG(NMEVENTOH, ', ') WITHIN GROUP (ORDER BY NMEVENTOH) DSEVENTOS
            FROM
                PTO_SITUFREQUENCIA SF
                LEFT JOIN
                PTO_SITUFREQPARAM SFP ON
                    SFP.NRORG IN (SF.NRORG, :P_NRORG_PADRAO) AND
                    SFP.NRSITUFREQUENCIA = SF.NRSITUFREQUENCIA AND
                    SFP.IDATIVO = 'S'
                LEFT JOIN
                FPA_EVENTOM EVM ON
                    EVM.NRORG IN (SFP.NRORG, :P_NRORG_PADRAO) AND
                    EVM.NREVENTOM = SFP.NREVENTOM
                LEFT JOIN
                FPA_EVENTOH EVH ON
                    EVH.NRORG = EVM.NRORG AND
                    EVH.NREVENTOM = EVM.NREVENTOM AND
                    EVH.DTMESCOMPETENC = (
                        SELECT
                            MAX(EVH1.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH EVH1
                        WHERE
                            EVH1.NRORG = EVM.NRORG AND
                            EVH1.NREVENTOM = EVM.NREVENTOM AND
                            EVH1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') AND
                            EVH1.IDATIVO = 'S'
                    )
            WHERE
                SF.NRORG IN (:P_NRORG, :P_NRORG_PADRAO) AND
                SF.IDATIVO = 'S' AND
                (
                    TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') >= SF.DTINIVIGENCIA OR
                    SF.DTINIVIGENCIA IS NULL
                ) AND
                (
                    TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') <= SF.DTFIMVIGENCIA OR
                    SF.DTFIMVIGENCIA IS NULL
                )
            GROUP BY
                SF.NRSITUFREQUENCIA,
                SF.NRORG,
                SF.DSSITUFREQUENCIA,
                SF.DSABREVIACAO
            ORDER BY
                SF.DSSITUFREQUENCIA
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function editRowResumoFreq(
        array $params
    ) {
        $sql = <<<SQL
            UPDATE
                PTO_RESUMOSITUFREQ
            SET
                NRSITUFREQUENCIA = :P_NRSITUFREQUENCIA,
                QTREFAPURSITUFREQ = :P_QTREFAPURSITUFREQ,
                CDOPERULTATU = :P_CDOPERADOR,
                DTULTATU = SYSDATE,
                NRORGULTATU = :P_NRORG
            WHERE
                NRORG = :P_NRORG AND
                NRRESUMOSITUFREQ = :P_NRRESUMOSITUFREQ
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function deleteRowResumoFreq(
        array $params
    ) {
        $sql = <<<SQL
            DELETE FROM
                PTO_RESUMOSITUFREQ
            WHERE
                NRORG = :P_NRORG AND
                NRRESUMOSITUFREQ = :P_NRRESUMOSITUFREQ
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function saveRowResumoFreq(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO PTO_RESUMOSITUFREQ (
                NRORG,                  NRRESUMOSITUFREQ,       NRPROGOCORRENCIA,
                NRPERIODOAPURACAO,      NRVINCULOM,             NRMOTIVOOCORFREQ,
                DTAPURACAOSITUFREQ,     IDFORCASITUFREQ,        QTREFAPURSITUFREQ,
                QTREFABONSITUFREQ,      QTREFAPUNOVASITUFREQ,   DSOBSERVACAO,
                NRLOTELANCEVENTO,       IDATIVO,                NRORGINCLUSAO,
                DTINCLUSAO,             CDOPERINCLUSAO,         NRORGULTATU,
                DTULTATU,               CDOPERULTATU,           NRSITUFREQNOVA,
                NRSITUFREQUENCIA,       IDCONFIRMADA
            ) VALUES (
                :NRORG,                 :NRRESUMOSITUFREQ,      NULL,
                :NRPERIODOAPURACAO,     :NRVINCULOM,            NULL,
                :DTAPURACAOSITUFREQ,    NULL,                   :QTREFAPURSITUFREQ,
                NULL,                   NULL,                   NULL,
                NULL,                   'S',                    :NRORG,
                SYSDATE,                :CDOPERADOR,            :NRORG,
                SYSDATE,                :CDOPERADOR,            NULL,
                :NRSITUFREQUENCIA,      'S'
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}