<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\{ DateUtil, Environment, NovoCodigo, Repositories };

use Ems\Services\EstruturaService;
use Ems\Repository\SolicitacaoRepository;
use Ems\Util\{ ArrayUtil, Service, Utilities };

use Ems\Logic\ProcessamentoSolicitacao\{ ProcessamentoSolicitacao, ProcessamentoSolicitacaoFactory };

use Teknisa\Model\Gestaopessoas\Entities\{ GpeFaseaprovsoli, GpeFasetramsoli, GpeSolicitacao };

class SolicitacaoService extends Service {

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var Environment */
    protected $environment;

    /** @var SolicitacaoRepository */
    protected $solicitacaoRepository;
    
    const IDAPROVACAOSEC_APROVADO       = 'A';
    const IDAPROVACAOSEC_PENDENTE       = 'P';
    const IDAPROVACAOSEC_REPROVADO      = 'R';
    const IDAPROVACAOSEC_EMANDAMENTO    = 'E';

    public function __construct(
        EntityManager           $entityManager,
        SolicitacaoRepository   $solicitacaoRepository,
        Environment             $environment,
        EstruturaService        $estruturaService
    ) {
        $this->entityManager            = $entityManager;
        $this->solicitacaoRepository    = $solicitacaoRepository;
        $this->environment              = $environment;
        $this->estruturaService         = $estruturaService;
    }
    
    public function getEntityRepositoryName(): string {
        return Repositories::GPE_SOLICITACAO;
    }

    public function getEntityKeysName(): array {
        return [
            'NRORG',
            'NRSOLICITACAO'
        ];
    }

    public function getSolicitacoesVinculosGestor(
        int     $nrEstrutura,
        ?string $cdOperadorAprovante = null
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes(new \Setforms2\Shared\ORM\DateTime())->format(DateUtil::FORMATO_BRASILEIRO);

        $nrEstruturasString = null;

        if ($nrEstrutura !== -1) {
            $nrEstruturas = $this->estruturaService->
                getEstruturasInferiores($nrEstrutura, true);

            $nrEstruturasString = ArrayUtil::toStringSQL($nrEstruturas);
        }

        $params = [
            'P_NRORG'                   => $nrOrg,
            'P_NRESTRUTURAS'            => $nrEstruturasString,
            'P_DTMESCOMPETENC'          => $dtmescompetenc,
            'P_CDOPERADOR_APROVANTE'    => $cdOperadorAprovante,
        ];

        $solicitacoes = $this->solicitacaoRepository->
            getSolicitacoesVinculosGestor($params);

        return $solicitacoes;
    }

    public function getItemSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getItemSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getTiposolicitacao(?int $nrTipoSolicitacao = null) {
        $nrorg = $this->environment->getNrorg();
        $tipos = $this->solicitacaoRepository->getTiposolicitacao($nrorg, $nrTipoSolicitacao);

        foreach($tipos as &$tipo) {
            if ($tipo['NRTIPOSOLICITACAO'] === '6') {
                $tipo['NMTIPOSOLICITACAO'] = 'Solicitação de Seleção de Pessoal';
            }
        }

        return $tipos;
    }

    public function getTipoRelacionamento() {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposRel = $this->solicitacaoRepository->getTipoRelacionamento(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposRel;
    }

    public function getLinhaTransporteAtiva(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('1/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetencia,
        ];

        $linhas = $this->solicitacaoRepository->getLinhaTransporteAtiva(
            $params
        );

        return $linhas;
    }

    public function getTipolinhavinc(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposLinhas = $this->solicitacaoRepository->getTipolinhavinc(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposLinhas;
    }

    public function getIntegtransp(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORG_PADRAO'        => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
        ];

        $numerosIntegracao = $this->solicitacaoRepository->getIntegtransp(
            $params
        );

        return $numerosIntegracao;
    }

    public function getSentidolinha(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $numerosIntegracao = $this->solicitacaoRepository->getSentidolinha(
            $nrOrg, $nrOrgPadrao
        );

        return $numerosIntegracao;
    }

    public function getBeneficio(): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $beneficios = $this->solicitacaoRepository->getBeneficio(
            $nrOrg, $nrOrgPadrao
        );

        return $beneficios;
    }

    public function getPessoaRelacionamentos(
        int $nrPessoa
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG'               => $nrOrg,
            'P_NRORGPADRAO'         => $nrOrgPadrao,
            'P_DTMESCOMPETENC'      => $dtMesCompetencia,
            'P_NRPESSOA'            => $nrPessoa,
            'P_NRTIPORELACIONA'     => null,
        ];

        $pessoas = $this->solicitacaoRepository->getPessoaRelacionamentos(
            $params
        );

        return $pessoas;
    }

    public function getTipoDependente(): array {
        $nrOrgPadrao = $this->environment->getNrOrgPadrao();

        $tiposDep = $this->solicitacaoRepository->getTipoDependente($nrOrgPadrao);

        return $tiposDep;
    }

    public function getTemplate(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $templates = $this->solicitacaoRepository->getTemplate($nrOrg);

        return $templates;
    }

    public function getTipoprocessos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getTipoprocessos($nrOrg);

        return $tipos;
    }

    public function getParcnegocioVinculos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $tipos = $this->solicitacaoRepository->getParcnegocioVinculos($nrOrg);

        return $tipos;
    }

    public function getTipoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tipos = $this->solicitacaoRepository->getTipoTransferencia(
            $nrOrg, $nrOrgPadrao
        );

        return $tipos;
    }

    public function getMovimentacaoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = date('01/m/Y');

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
            'P_DTMESCOMPETENC' => $dtMesCompetencia,
        ];

        $tipos = $this->solicitacaoRepository->getMovimentacaoTransferencia(
            $params
        );

        return $tipos;
    }

    public function getMotivoTransferencia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $motivos = $this->solicitacaoRepository->getMotivoTransferencia(
            $nrOrg
        );

        return $motivos;
    }
    
    public function aprovaSolicitacao(
        string  $nrSolicitacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = []
    ): bool {
        return $this->atualizaSolicitacao(
            $nrSolicitacao,
            self::IDAPROVACAOSEC_APROVADO,
            $cdOperadorAprovante,
            $dsObservacao,
            $extraRow
        );
    }
    
    public function reprovaSolicitacao(
        string  $nrSolicitacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = []
    ): bool {
        return $this->atualizaSolicitacao(
            $nrSolicitacao,
            self::IDAPROVACAOSEC_REPROVADO,
            $cdOperadorAprovante,
            $dsObservacao,
            $extraRow
        );
    }
    
    /**
     * Atualiza o status e as fases de aprovacao de solicitacao. Caso a ultima fase for aprovada procede com as insercoes de acordo com as entidades ProcessmentoSolicitacao.
     * 
     * @param string  $nrSolicitacao
     * @param string  $idAprovacao
     * @param string  $cdOperadorAprovante
     * @param ?string $dsObservacao
     * @param ?array  $extraRow
     * 
     * @return bool
     */
    public function atualizaSolicitacao(
        string  $nrSolicitacao,
        string  $idAprovacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao           = null,
        ?array  $extraRow               = []
    ): bool {
        $solicitacao = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->findOneBy(array('nrorg' => $this->environment->getNrOrg(), 'nrsolicitacao' => $nrSolicitacao));
        if (empty($solicitacao)) throw new \Exception('Solicitação não encontrada.');
        
        $erro = $this->validaModeloAprovacao($solicitacao->getNrtiposolicitacao());
                
        if (!$erro['Bloquear']) {

            $idaprovacao = $this->aprovaFaseTramitacao($solicitacao->getNrsolicitacao(), $solicitacao->getNrtiposolicitacao(), $idAprovacao, $cdOperadorAprovante, $dsObservacao);
            $faseComAprovacaoNaoNecessaria = empty($idaprovacao);
            if ($faseComAprovacaoNaoNecessaria) {
                return true;
            }
            
            $processamentoSolicitacao = $this->getProcessamentoSolicitacao($solicitacao, $extraRow);
            if (!empty($processamentoSolicitacao)) {
                if ($idaprovacao === 'A') {
                    $processamentoSolicitacao->efetivar();
                } else {
                    $processamentoSolicitacao->rejeitar();
                }
            }

            $this->update(
                [
                    'NRORG'             => $this->environment->getNrorg(),
                    'NRSOLICITACAO'     => $nrSolicitacao,
                    'IDAPROVACAOSEC'    => $idaprovacao,
                    'CDOPERULTATU'      => $cdOperadorAprovante,
                ]
            );
            
            return true;
        }
        
        throw new \Exception($erro['Texto']);
    }
    
    /**
     * Valida a existencia de um modelo para um determinado tipo de solicitacao.
     * 
     * @param $nrtiposolicitacao
     * 
     * @return array
     */
    public function validaModeloAprovacao($nrtiposolicitacao) {
        $erro["Bloquear"] = false;
        
        $perfil = $this->validaPerfildeAcesso($nrtiposolicitacao);
        if(!$perfil) {
            $erro["Texto"] = "Não existe perfil de acesso para este tipo de solicitação.";
            $erro["Bloquear"] = true;
        } else {
            if(!$perfil->getNrmodeloaprovsoli()) {
                $erro["Texto"] = "Modelo de aprovação não parametrizado para este tipo de solicitação.";
                $erro["Bloquear"] = true;
            }
        }

        return $erro;
    }
    
    /**
     * Busca o registro de GPE_PERFACESTIPOSOLI associado ao tipo de solicitacao.
     * 
     * @param $tiposoli
     * 
     * @return ?GpePerfacestiposoli
     */
    public function validaPerfildeAcesso($tiposoli){
        $perfil = $this->entityManager->getRepository(Repositories::GPE_PERFACESTIPOSOLI)->findOneBy(array('nrorg'=>$this->environment->getNrorg(),'nrtiposolicitacao'=>$tiposoli));
        if (!$perfil) {
            return false;
        }
        return $perfil;
    }
    
    /**
     * Retorna o proximo IDAPROVACAOSEC a ser atualizado na solicitacao. Caso nao existam proximas fases, null eh retornado.
     * 
     * @param string  $nrSolicitacao
     * @param string  $nrTipoSolicitacao
     * @param string  $idAprovacao
     * @param string  $cdOperadorAprovante
     * @param ?string $dsObservacao
     * 
     * @return ?string
     */
    public function aprovaFaseTramitacao(
        string  $nrSolicitacao,
        string  $nrTipoSolicitacao,
        string  $idAprovacao,
        string  $cdOperadorAprovante,
        ?string $dsObservacao = null
    ): ?string {
        $faseatual = $this->entityManager->getRepository(Repositories::GPE_FASETRAMSOLI)->retornaFaseatual($this->environment->getNrorg(), $nrSolicitacao, Repositories::class);

        if ($faseatual) {
            $nrordem = $faseatual->getNrordem();
        } else {
            $nrordem = 0;
        }

        $proximafase = $this->entityManager->getRepository(Repositories::GPE_PERFACESTIPOSOLI)->retornaProximaFase($this->environment->getNrorg(), $nrTipoSolicitacao, $nrordem);
        $naoExisteProximaFase = empty($proximafase);
        if ($naoExisteProximaFase) {
            return null;
        }
        
        $perfilfase = explode(',',$proximafase->getDsnrperfilacesso());

        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('cdoperador'=>$cdOperadorAprovante));

        $perfiloper = $this->entityManager->getRepository(Repositories::PERFACESSPARC)->findBy(
            array('nrorg'=>$this->environment->getNrorg(), 'nrparcnegocio'=>$operador->getNrparcnegocio(), 'nrperfilacesso'=>$perfilfase)
        );

        if ($perfiloper) {
            if ($idAprovacao == 'A') {
                $idaprovado = 'S';
            } else {
                $idaprovado = 'N';
            }

            $fasetram = $this->insereFaseTramSoli($nrSolicitacao, $proximafase->getNrfaseaprovsoli(), $idaprovado, $cdOperadorAprovante, $dsObservacao);

            $ultimafase = $this->entityManager->getRepository(Repositories::GPE_FASEAPROVSOLI)->findOneBy(
                array('nrorg' => $this->environment->getNrorg(), 'nrmodeloaprovsoli' => $proximafase->getNrmodeloaprovsoli()),
                array('nrordem' => 'DESC')
            );

            /*
            Se for aprovar e a fase for a última do modelo, muda o status da solicitação para aprovada
            Se for aprovar e não for a última fase o status da solicitação fica como em andamento
            Se for reprovar e a fase for obrigatória muda o status da solicitação para reprovada
            Se for reprovar e a fase não for obrigatória o status da solicitação fica como em andamento
            */
            $novoIdAprovacao = null;
            
            if ($idAprovacao == 'A') {
                if ($proximafase->getNrfaseaprovsoli() == $ultimafase->getNrfaseaprovsoli()) {
                    $novoIdAprovacao = 'A';
                } else {
                    $novoIdAprovacao = 'E';
                }
            } else {
                if ($proximafase->getIdfaseobrigatoria() == 'S') {
                    $novoIdAprovacao = 'R';
                } else {
                    $novoIdAprovacao = 'E';
                }
            }

            return $novoIdAprovacao;
        } else {
            throw new \Exception("Operador não perfil de acesso para aprovar essa fase da solicitação", 10);
        }
    }
    
    /**
     * Inclui o registro de uma fase para uma solicitacao.
     * 
     * @param string  $nrSolicitacao,
     * @param string  $nrFaseAprovSoli,
     * @param string  $idAprovado,
     * @param string  $cdOperadorAprovante,
     * @param ?string $dsObservacao = null
     * 
     * @return GpeFasetramsoli
     */
    public function insereFaseTramSoli(
        string  $nrSolicitacao,
        string  $nrFaseAprovSoli,
        string  $idAprovado,
        string  $cdOperadorAprovante,
        ?string $dsObservacao = null
    ): GpeFasetramsoli {

        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeFasetramsoli $gpe_fasetramsoli */
        $gpe_fasetramsoli = Repositories::GPE_FASETRAMSOLI;    
        
        $faseTram = new $gpe_fasetramsoli();

        $faseTram->setNrsolicitacao($nrSolicitacao);
        $faseTram->setNrfaseaprovsoli($nrFaseAprovSoli);
        $faseTram->setIdaprovado($idAprovado);
        $faseTram->setDsobservacao($dsObservacao);
        $faseTram->setNrorg($this->environment->getNrorg());
        $faseTram->setDtinclusao(DateUtil::getDataAtual());
        $faseTram->setCdoperinclusao($cdOperadorAprovante);
        $faseTram->setNrorginclusao($this->environment->getNrorg());
        
        $nrFaseTramSoli = NovoCodigo::geraCodigo($this->environment->getNrorg(), 'GPE_FASETRAMSOLI', 1, 12, true);
        $faseTram->setNrfasetramsoli($nrFaseTramSoli);
        
        $this->entityManager->persist($faseTram);
        $this->entityManager->flush();
        
        return $faseTram;
    }
    
    /**
     * Busca a entidade de processamento da aprovacao de uma solicitacao capazes de efetiva-las.
     * 
     * @param GpeSolicitacao  $solicitacao,
     * @param ?array          $extraRow
     * 
     * @return ?ProcessamentoSolicitacao
     */
    public function getProcessamentoSolicitacao(
        GpeSolicitacao  $solicitacao,
        ?array          $extraRow
    ): ?ProcessamentoSolicitacao {
        return ProcessamentoSolicitacaoFactory::createProcessamentoSolicitacao($solicitacao, $extraRow);
    }
    
    public static function isTipoSoliOnlyEmail(string $tipoSoli): bool {
        return $tipoSoli < 1;
    }
}
